const W = 60;
const H = 22;;

let board = [];
for (let i=0; i<W; i++) {
    board[i] = [];
    for (let j=0; j<H; j++) {
        board[i][j] = false;
    }
};

const NORTH = {x: 0, y: 1};
const SOUTH = {x: 0, y: -1};
const EAST  = {x: 1, y: 0};
const WEST  = {x: -1, y: 0};

const directions = [NORTH, SOUTH, EAST, WEST];;

function pointAdd(a, b) {
    return {x: (a.x + b.x), y: (a.y + b.y)};
};

function getBoardPoint(board, point) {
    return board[point.x][point.y];
};

function boolToInt(val) {
    return (val) ? 1 : 0;
}

function add(a, b) {
    return a + b;
}

function countBranches(board, point) {
    return directions
        .map(d => pointAdd(d, point))
        .map(p => getBoardPoint(board, p))
        .map(boolToInt)
        .reduce(add);
}

function isValidPath(board, point) {
    const {x, y} = point;
    return (x > 0) && (y > 0) && (x < W-1) && (y < H-1)
        && (board[x][y] === false)
        && (countBranches(board, point) <= 1);
};

function shuffle (a) {
  let j = 0,
      temp = null,
      array = a.slice(0); // a.slice(0) takes a copy of the array.

  for (let i = array.length - 1; i > 0; i -= 1) {
    j = Math.floor(Math.random() * (i + 1))
    temp = array[i]
    array[i] = array[j]
    array[j] = temp
  }
  return array;
};

const START = {x: 0, y: 1};
let   branchPoints = [START];
board[START.x][START.y] = true;;

let pointToTest = null;
let directionsToTest;
let direction;
let newBranchPoint;
let newPointFound = false;
while (branchPoints.length > 0) {
    i = Math.floor(Math.random() * Math.floor(branchPoints.length - 1));
    pointToTest = branchPoints[i];
    directionsToTest = shuffle(directions);
    newPointFound = false;
    while (!newPointFound && directionsToTest.length > 0) {
        direction = directionsToTest.shift();
        newBranchPoint = pointAdd(pointToTest, direction);
        if (isValidPath(board, newBranchPoint)) {
            branchPoints.push(newBranchPoint);
            board[newBranchPoint.x][newBranchPoint.y] = true;
            newPointFound = true;
        }
    }
    if (!newPointFound) {
        // Delete item from branchPoints array
        branchPoints.splice(i, 1);
    }
};

function addExit(board) {
    var j = H - 1;
    while ((board[W - 2][j] === false) && j >= 0) {
        j -= 1;
    }
    if (j >= 0) {
        board[W -1][j] = true;
    }
    return board;
}

board = addExit(board);;

function transposeBoard(board) {
    let newBoard = [];
    for (let j=0; j<H; j++) {
        newBoard[j] = [];
    }
    for (let i=0; i<W; i++) {
        for(let j=0; j<H; j++) {
            newBoard[j][i] = board[i][j];
        }
    }
    return newBoard;
}

const transposedBoard = transposeBoard(board);;

function boolToMazeChar(b) {
    return (b) ? '░' : '▓';
}

function boardToString(board) {
    return board.map(col => col.map(boolToMazeChar).join('')).join("\n");
}
console.log(boardToString(transposedBoard));
;